#include "StdAfx.h"
#include "ChildView.h"
#include "ship.h"
#include "bullet.h"
#include "vector.h"
#include <math.h>

#define MAX_VELOCITY	10
extern int randInt(int min, int max);
extern int MAX_SHIPS;

ship::ship()
{
	hit = false;
	hp = 25;
	Dir.ex = 0.0;
	Dir.ey = 0.0;
	Pt = Dir;
	Pt.ey+=2;
	Pos.ex = randInt(1, 500);
	Pos.ey = randInt(1, 500);
	score = 0;
	pw = &G_SINGLEFIRE;
	ai = true;
	shipID = -1;
	invCount = 0;
	tabooCount = 0;
	deaths = 0;
	coolDown = 0;
	target = 0;
}

void ship::spawn(double sposx, double sposy, double sdirx, 
		double sdiry, double sptx, double spty)
{
	hit = false;
	Pos.ex = sposx;
	Pos.ey = sposy;
	Dir.ex = sdirx;
	Dir.ey = sdiry;
	Pt.ex = sptx;
	Pt.ey = spty;
	invCount = 15;
	hp = 25;
	if(tabooCount < 15)
		tabooCount = 15;
	if(ai)
		target = aiPickTarget();
}

void ship::accelerate(int direction)
{

	if(Dir.length() > 5 && !tabooCount)
		Dir.setLength(5);
	else if(Dir.length() > 10 && tabooCount)
		Dir.setLength(10);
	
	Dir += Pt * direction * 0.15;
}

void ship::turnLeft()
{ Pt.rotate(-6); }

void ship::turnRight()
{ Pt.rotate(6); }

void ship::Paint(CDC &rdc)
{	
    CPen* pOldPen;

	double cx; double cy;
	double l = Pt.length();
	double angl = Pt.angle()-30;

	CString health;
	health.Format("%d", hp);
	rdc.TextOut(Pos.ex-4, Pos.ey+15, health);

	pOldPen=rdc.SelectObject(&myPen);

	rdc.MoveTo(Pos);		//Go to the middle of ship
	rdc.LineTo(Pos+Pt);		//Draw the bar
	rdc.LineTo(Pos - Pt);	//down the middle
	rdc.MoveTo(Pos+Pt);		//Go to pointy front of ship

//Right wing
	cx = (Pt.ex - 15 * cos(Pt.dtor(angl)));
	cy = (Pt.ey - 15 * sin(Pt.dtor(angl)));
	rdc.LineTo(int(cx+Pos.ex-Pt.ex), int(cy+Pos.ey-Pt.ey));
	rdc.LineTo(Pos - Pt);
// Left wing
	angl -= 115;
	cx = (Pt.ex + 15 * cos(Pt.dtor(angl)));
	cy = (Pt.ey + 15 * sin(Pt.dtor(angl)));
	rdc.MoveTo(Pos+Pt);
	rdc.LineTo(cx+Pos.ex-Pt.ex, cy+Pos.ey-Pt.ey);
	rdc.MoveTo(cx+Pos.ex-Pt.ex, cy+Pos.ey-Pt.ey);
	rdc.LineTo(Pos - Pt);
//Invincibility
	if(invCount)
	{
		rdc.MoveTo(Pos.ex + 10, Pos.ey + 14);
		rdc.LineTo(Pos.ex + 14, Pos.ey + 10);
		rdc.MoveTo(Pos.ex + 14, Pos.ey - 10);
		rdc.LineTo(Pos.ex + 10, Pos.ey - 14);
		rdc.MoveTo(Pos.ex - 14, Pos.ey - 10);
		rdc.LineTo(Pos.ex - 10, Pos.ey - 14);
		rdc.MoveTo(Pos.ex - 10, Pos.ey + 14);
		rdc.LineTo(Pos.ex - 14, Pos.ey + 10);
	}
//taboo
	if(tabooCount)
	{
		rdc.MoveTo(Pos.ex + 14, Pos.ey - 10);
		rdc.LineTo(Pos.ex + 14, Pos.ey + 10);
		rdc.MoveTo(Pos.ex - 10, Pos.ey - 14);
		rdc.LineTo(Pos.ex + 10, Pos.ey - 14);
		rdc.MoveTo(Pos.ex - 14, Pos.ey + 10);
		rdc.LineTo(Pos.ex - 14, Pos.ey - 10);
		rdc.MoveTo(Pos.ex - 10, Pos.ey + 14);
		rdc.LineTo(Pos.ex + 10, Pos.ey + 14);
	}
	if(hit)
	{
		CBrush pBrush;
		CBrush* pOldBrush;
		pBrush.CreateSolidBrush(RGB(200, 200, 50));
		pOldBrush = rdc.SelectObject(&pBrush);
		hit = false;
		int offsetx = randInt(0, 20) - 10;
		int offsety = randInt(0, 20) - 10;
		rdc.Ellipse(Pos.ex+offsetx-6, Pos.ey+offsety-6, Pos.ex+offsetx+6, Pos.ey+offsety+6);
		rdc.SelectObject(pOldBrush);
		pBrush.DeleteObject();
	}

	for(int i=0; i<10; i++)
		b[i].Paint(rdc);
	rdc.SelectObject(pOldPen);
}

void ship::tick(CRect &rc)
{
	Pos +=Dir;
	Pos %= rc.Size();

	if(Dir.length() > 5 && !tabooCount)
		Dir.setLength(5);

	for(int i=0; i<10; i++)
	{
		if(b[i].active)
			b[i].onTick(rc);
	}
	coolDown--;
}


void ship::reset(int bullID)
{
	b[bullID].active = true;
	b[bullID].Dir = Pt * 3 + Dir;
	b[bullID].Pos = Pos;
	b[bullID].ttl = MAX_MISSILE_LIFE;
	b[bullID].active = true;
	numBulls++;
}

int ship::giveFreeBulletID()
{
	for(int i=0; i<10; i++)
	{
		if(!b[i].active)
			return i;
	}
	return -1;
}

void ship::aitarget(CFlyingObject &target)
{
	VECTOR sPos = Pos;
	VECTOR sPt = target.Pos + target.Dir;
	VECTOR temp = sPt - sPos;
	int freeBullet = giveFreeBulletID();
    
	double seAngle = Pt.angle();
	if(seAngle < 0)
		seAngle += 360;
	
	double angl = seAngle - temp.angle();
	angl = int(angl + 360) % 360;

	if(angl > -6 && angl < 6 && freeBullet >= 0)
	{
  		pw->fire(*this); 
		accelerate(1);
	}
	else if(angl > 180)
		Pt.rotate(5);
	else if(angl < 180)
		Pt.rotate(-5);

	if(pw == &G_RINGOFDOOM)
	{
  		pw->fire(*this); 
		accelerate(1);
	}
}

int ship::aiPickTarget()
{
	int index = randInt(0, 6);
	return index;
}

void ship::CreateMyPen(COLORREF color)
{
	myPen.CreatePen(PS_SOLID, 1, color);
	myColor = color;
}